#include <string.h>
#include "keyboard.h"
#include "sysvia.h"

int gp2x_key_lookup[9];

int gp2x_mapping[9][2] =
 {
         {GP2X_BUTTON_LEFT, SDLK_z},
         {GP2X_BUTTON_RIGHT,SDLK_x},
         {GP2X_BUTTON_UP, SDLK_QUOTE},
         {GP2X_BUTTON_DOWN,SDLK_SLASH},
         {GP2X_BUTTON_A,SDLK_SPACE},
         {GP2X_BUTTON_B,SDLK_RETURN},
         {GP2X_BUTTON_X,SDLK_n},
         {GP2X_BUTTON_Y,SDLK_y},
         {GP2X_BUTTON_SELECT, SDLK_F10}
 };


static struct BeebKeyTrans SDLtoBeebEmKeymap[]={
// SDL          BBC     BBC KEY NAME (see doc/keyboard.jpg)
{SDLK_TAB,              6,0},    // TAB
{SDLK_RETURN,           4,9},    // RETURN

{SDLK_LCTRL,            0,1},    // CONTROL
{SDLK_RCTRL,            0,1},    // CONTROL

{SDLK_LSHIFT,           0,0},   // SHIFT
{SDLK_RSHIFT,           0,0},   // SHIFT

{SDLK_CAPSLOCK,         4,0},    // CAPS LOCK (Totally fucked up in SDL..)
{SDLK_LSUPER,           4,0},   // CAPS LOCK (so Alt Gr is also CAPS-LOCK..)

{SDLK_ESCAPE,           7,0},    // ESCAPE
{SDLK_SPACE,            6,2},    // SPACE

{SDLK_LEFT,             1,9},    // LEFT
{SDLK_UP,               3,9},    // UP
{SDLK_RIGHT,            7,9},    // RIGHT
{SDLK_DOWN,             2,9},    // DOWN

{SDLK_DELETE,           5,9},    // DELETE
{SDLK_BACKSPACE,        5,9},    // DELETE

{SDLK_INSERT,           6,9},   // COPY

{SDLK_0,                2,7},    // 0
{SDLK_1,                3,0},    // 1
{SDLK_2,                3,1},    // 2
{SDLK_3,                1,1},    // 3
{SDLK_4,                1,2},    // 4
{SDLK_5,                1,3},    // 5
{SDLK_6,                3,4},    // 6
{SDLK_7,                2,4},    // 7
{SDLK_8,                1,5},    // 8
{SDLK_9,                2,6},    // 9
{SDLK_a,                4,1},    // A
{SDLK_b,                6,4},    // B
{SDLK_c,                5,2},    // C
{SDLK_d,                3,2},    // D
{SDLK_e,                2,2},    // E
{SDLK_f,                4,3},    // F
{SDLK_g,                5,3},    // G
{SDLK_h,                5,4},    // H
{SDLK_i,                2,5},    // I
{SDLK_j,                4,5},    // J
{SDLK_k,                4,6},    // K
{SDLK_l,                5,6},    // L
{SDLK_m,                6,5},    // M
{SDLK_n,                5,5},    // N
{SDLK_o,                3,6},    // O
{SDLK_p,                3,7},    // P
{SDLK_q,                1,0},    // Q
{SDLK_r,                3,3},    // R
{SDLK_s,                5,1},    // S
{SDLK_t,                2,3},    // T
{SDLK_u,                3,5},    // U
{SDLK_v,                6,3},    // V
{SDLK_w,                2,1},    // W
{SDLK_x,                4,2},    // X
{SDLK_y,                4,4},    // Y
{SDLK_z,                6,1},    // Z
{SDLK_F10,              2,0},    // f0
{SDLK_F1,               7,1},    // f1
{SDLK_F2,               7,2},    // f2
{SDLK_F3,               7,3},    // f3
{SDLK_F4,               1,4},    // f4
{SDLK_F5,               7,4},    // f5
{SDLK_F6,               7,5},    // f6
{SDLK_F7,               1,6},    // f7
{SDLK_F8,               7,6},    // f8
{SDLK_F9,               7,7},    // f9

{SDLK_MINUS,            5,7},   // "+" / ";"
{SDLK_COMMA,            6,6},   // "<" / ","
{SDLK_EQUALS,           1,7},   // "=" / "-"
{SDLK_PERIOD,           6,7},   // ">" / "."
{SDLK_BACKQUOTE,        2,8},   // "-" / "£"

{SDLK_SEMICOLON,        4,7},   // "@"
{SDLK_QUOTE,            4,8},   // "*" / ":"

{SDLK_SLASH,            6,8},   // "/" / "?"

{SDLK_HASH,             1,8},   // circumflex / tilde

{SDLK_PAUSE,            -2,-2}, // BREAK

{-1, -1, -1}              // ** END OF LIST **
};

int ConvertSDLKeyToBBCKey(int sym, int *col, int *row)
{
        struct BeebKeyTrans *p = SDLtoBeebEmKeymap;

        for(;((p->row != -1) && (p->sym != sym));p++) {}

        *(row) = p->row;
        *(col) = p->col;

        return *row<0 ? 0 : 1;
}

int apply_mapping(int gp2xButton, int sdlk)
{
        for (int i=0; i < 9; i++)
        {
                //printf("ZZZZZZZ %d\n", gp2x_mapping[i][0]);
                if (gp2x_mapping[i][0] == gp2xButton)
                {
                        //printf("apply_mapping gp2xButton = %d   [%d][1] = %d\n", gp2xButton, i, sdlk);
                        gp2x_mapping[i][1] = sdlk;
                }
        }
}

int gp2xButtonToSDLK(int button)
{
        for (int i = 0; i < 9; i++)
        {
                if (gp2x_mapping[i][0] == button)
                {
                        //printf ("XXX button = %d, sdlk = %d\n", button, gp2x_mapping[i][1]);
                        return gp2x_mapping[i][1];
                }
                else
                {
                        //printf ("not found: %d tried against %d\n", button, gp2x_mapping[i][0]);
                }
        }
}

int buttonnameToGP2XButton(char* name)
{
        if (!strcmp (name, "LEFT")) return GP2X_BUTTON_LEFT;
        if (!strcmp (name, "RIGHT")) return GP2X_BUTTON_RIGHT;
        if (!strcmp (name, "UP")) return GP2X_BUTTON_UP;
        if (!strcmp (name, "DOWN")) return GP2X_BUTTON_DOWN;
        if (!strcmp (name, "A")) return GP2X_BUTTON_A;
        if (!strcmp (name, "B")) return GP2X_BUTTON_B;
        if (!strcmp (name, "X")) return GP2X_BUTTON_X;
        if (!strcmp (name, "Y")) return GP2X_BUTTON_Y;
        if (!strcmp (name, "SELECT")) return GP2X_BUTTON_SELECT;
        return -1;
}

char* SDLKToKeyname(int sdlk)
{
        if (sdlk == SDLK_BACKSPACE) return "BACKSPACE";
        if (sdlk == SDLK_TAB) return "TAB";
        if (sdlk == SDLK_CLEAR) return "CLEAR";
        if (sdlk == SDLK_ESCAPE) return "ESCAPE";
        if (sdlk == SDLK_SPACE) return "SPACE";
        if (sdlk == SDLK_EXCLAIM) return "EXCLAIM";
        if (sdlk == SDLK_QUOTEDBL) return "QUOTEDBL";
        if (sdlk == SDLK_HASH) return "HASH";
        if (sdlk == SDLK_PAUSE) return "PAUSE";
        if (sdlk == SDLK_DOLLAR) return "DOLLAR";
        if (sdlk == SDLK_AMPERSAND) return "AMPERSAND";
        if (sdlk == SDLK_QUOTE) return "QUOTE";
        if (sdlk == SDLK_LEFTPAREN) return "LEFTPAREN";
        if (sdlk == SDLK_RIGHTPAREN) return "RIGHTPAREN";
        if (sdlk == SDLK_ASTERISK) return "ASTERISK";
        if (sdlk == SDLK_COMMA) return "COMMA";
        if (sdlk == SDLK_MINUS) return "MINUS";
        if (sdlk == SDLK_PERIOD) return "PERIOD";
        if (sdlk == SDLK_SLASH) return "SLASH";
        if (sdlk == SDLK_0) return "0";
        if (sdlk == SDLK_1) return "1";
        if (sdlk == SDLK_2) return "2";
        if (sdlk == SDLK_3) return "3";
        if (sdlk == SDLK_4) return "4";
        if (sdlk == SDLK_5) return "5";
        if (sdlk == SDLK_6) return "6";
        if (sdlk == SDLK_7) return "7";
        if (sdlk == SDLK_8) return "8";
        if (sdlk == SDLK_9) return "9";
        if (sdlk == SDLK_QUOTE) return "COLON";
        if (sdlk == SDLK_SEMICOLON) return "SEMICOLON";
        if (sdlk == SDLK_LESS) return "LESS";
        if (sdlk == SDLK_EQUALS) return "EQUALS";
        if (sdlk == SDLK_GREATER) return "GREATER";
        if (sdlk == SDLK_QUESTION) return "QUESTION";
        if (sdlk == SDLK_AT) return "AT";
        if (sdlk == SDLK_LEFTBRACKET) return "LEFTBRACKET";
        if (sdlk == SDLK_BACKSLASH) return "BACKSLASH";
        if (sdlk == SDLK_RIGHTBRACKET) return "RIGHTBRACKET";
        if (sdlk == SDLK_CARET) return "CARET";
        if (sdlk == SDLK_UNDERSCORE) return "UNDERSCORE";
        if (sdlk == SDLK_BACKQUOTE) return "BACKQUOTE";
        if (sdlk == SDLK_a) return "a";
        if (sdlk == SDLK_b) return "b";
        if (sdlk == SDLK_c) return "c";
        if (sdlk == SDLK_d) return "d";
        if (sdlk == SDLK_e) return "e";
        if (sdlk == SDLK_f) return "f";
        if (sdlk == SDLK_g) return "g";
        if (sdlk == SDLK_h) return "h";
        if (sdlk == SDLK_i) return "i";
        if (sdlk == SDLK_j) return "j";
        if (sdlk == SDLK_k) return "k";
        if (sdlk == SDLK_l) return "l";
        if (sdlk == SDLK_m) return "m";
        if (sdlk == SDLK_n) return "n";
        if (sdlk == SDLK_o) return "o";
        if (sdlk == SDLK_p) return "p";
        if (sdlk == SDLK_q) return "q";
        if (sdlk == SDLK_r) return "r";
        if (sdlk == SDLK_s) return "s";
        if (sdlk == SDLK_t) return "t";
        if (sdlk == SDLK_u) return "u";
        if (sdlk == SDLK_v) return "v";
        if (sdlk == SDLK_w) return "w";
        if (sdlk == SDLK_x) return "x";
        if (sdlk == SDLK_y) return "y";
        if (sdlk == SDLK_z) return "z";
        if (sdlk == SDLK_DELETE) return "DELETE";
        if (sdlk == SDLK_KP0) return "KP0";
        if (sdlk == SDLK_KP1) return "KP1";
        if (sdlk == SDLK_KP2) return "KP2";
        if (sdlk == SDLK_KP3) return "KP3";
        if (sdlk == SDLK_KP4) return "KP4";
        if (sdlk == SDLK_KP5) return "KP5";
        if (sdlk == SDLK_KP6) return "KP6";
        if (sdlk == SDLK_KP7) return "KP7";
        if (sdlk == SDLK_KP8) return "KP8";
        if (sdlk == SDLK_KP9) return "KP9";
        if (sdlk == SDLK_KP_PERIOD) return "KP_PERIOD";
        if (sdlk == SDLK_KP_DIVIDE) return "KP_DIVIDE";
        if (sdlk == SDLK_KP_MULTIPLY) return "KP_MULTIPLY";
        if (sdlk == SDLK_KP_MINUS) return "KP_MINUS";
        if (sdlk == SDLK_KP_PLUS) return "KP_PLUS";
        if (sdlk == SDLK_KP_ENTER) return "KP_ENTER";
        if (sdlk == SDLK_KP_EQUALS) return "KP_EQUALS";
        if (sdlk == SDLK_UP) return "UP";
        if (sdlk == SDLK_DOWN) return "DOWN";
        if (sdlk == SDLK_RIGHT) return "RIGHT";
        if (sdlk == SDLK_LEFT) return "LEFT";
        if (sdlk == SDLK_INSERT) return "INSERT";
        if (sdlk == SDLK_HOME) return "HOME";
        if (sdlk == SDLK_END) return "END";
        if (sdlk == SDLK_PAGEUP) return "PAGEUP";
        if (sdlk == SDLK_PAGEDOWN) return "PAGEDOWN";
        if (sdlk == SDLK_F10) return "F0";  // NB
        if (sdlk == SDLK_g) return "g";
        if (sdlk == SDLK_g) return "g";
        if (sdlk == SDLK_g) return "g";
        if (sdlk == SDLK_g) return "g";
        if (sdlk == SDLK_g) return "g";
        if (sdlk == SDLK_g) return "g";
        if (sdlk == SDLK_F1) return "F1";
        if (sdlk == SDLK_F2) return "F2";
        if (sdlk == SDLK_F3) return "F3";
        if (sdlk == SDLK_F4) return "F4";
        if (sdlk == SDLK_F5) return "F5";
        if (sdlk == SDLK_F6) return "F6";
        if (sdlk == SDLK_F7) return "F7";
        if (sdlk == SDLK_F8) return "F8";
        if (sdlk == SDLK_F9) return "F9";
        if (sdlk == SDLK_NUMLOCK) return "NUMLOCK";
        if (sdlk == SDLK_CAPSLOCK) return "CAPSLOCK";
        if (sdlk == SDLK_RSHIFT) return "RSHIFT";
        if (sdlk == SDLK_LSHIFT) return "LSHIFT";
        if (sdlk == SDLK_RCTRL) return "RCTRL";
        if (sdlk == SDLK_LCTRL) return "LCTRL";
        if (sdlk == SDLK_RALT) return "RALT";
        if (sdlk == SDLK_LALT) return "LALT";
        if (sdlk == SDLK_RMETA) return "RMETA";
        if (sdlk == SDLK_LMETA) return "LMETA";
        if (sdlk == SDLK_LSUPER) return "LSUPER";
        if (sdlk == SDLK_RSUPER) return "RSUPER";
        if (sdlk == SDLK_MODE) return "MODE";
        if (sdlk == SDLK_HELP) return "HELP";
        if (sdlk == SDLK_PRINT) return "PRINT";
        if (sdlk == SDLK_SYSREQ) return "SYSREQ";
        if (sdlk == SDLK_BREAK) return "BREAK";
        if (sdlk == SDLK_MENU) return "MENU";
        if (sdlk == SDLK_POWER) return "POWER";
        if (sdlk == SDLK_EURO) return "EURO";

        return "UNKNOWN";
}

int keynameToSDLK( char* name)
{
        if( !strcmp (name, "BACKSPACE")) return SDLK_BACKSPACE;
        if( !strcmp (name, "TAB")) return SDLK_TAB;
        if( !strcmp (name, "CLEAR")) return SDLK_CLEAR;
        if( !strcmp (name, "RETURN")) return SDLK_RETURN;
        if( !strcmp (name, "ESCAPE")) return SDLK_ESCAPE;
        if( !strcmp (name, "SPACE")) return SDLK_SPACE;
        if( !strcmp (name, "EXCLAIM")) return SDLK_EXCLAIM;
        if( !strcmp (name, "QUOTEDBL")) return SDLK_QUOTEDBL;
        if( !strcmp (name, "HASH")) return SDLK_HASH;
        if( !strcmp (name, "PAUSE")) return SDLK_PAUSE;
        if( !strcmp (name, "DOLLAR")) return SDLK_DOLLAR;
        if( !strcmp (name, "AMPERSAND")) return SDLK_AMPERSAND;
        if( !strcmp (name, "QUOTE")) return SDLK_QUOTE;
        if( !strcmp (name, "LEFTPAREN")) return SDLK_LEFTPAREN;
        if( !strcmp (name, "RIGHTPAREN")) return SDLK_RIGHTPAREN;
        if( !strcmp (name, "ASTERISK")) return SDLK_ASTERISK;
        if( !strcmp (name, "PLUS")) return SDLK_PLUS;
        if( !strcmp (name, "COMMA")) return SDLK_COMMA;
        if( !strcmp (name, "MINUS")) return SDLK_MINUS;
        if( !strcmp (name, "PERIOD")) return SDLK_PERIOD;
        if( !strcmp (name, "SLASH")) return SDLK_SLASH;
        if( !strcmp (name, "0")) return SDLK_0;
        if( !strcmp (name, "1")) return SDLK_1;
        if( !strcmp (name, "2")) return SDLK_2;
        if( !strcmp (name, "3")) return SDLK_3;
        if( !strcmp (name, "4")) return SDLK_4;
        if( !strcmp (name, "5")) return SDLK_5;
        if( !strcmp (name, "6")) return SDLK_6;
        if( !strcmp (name, "7")) return SDLK_7;
        if( !strcmp (name, "8")) return SDLK_8;
        if( !strcmp (name, "9")) return SDLK_9;
        if( !strcmp (name, "COLON")) return SDLK_QUOTE;  // dodgy mapping
        if( !strcmp (name, "SEMICOLON")) return SDLK_SEMICOLON;
        if( !strcmp (name, "LESS")) return SDLK_LESS;
        if( !strcmp (name, "EQUALS")) return SDLK_EQUALS;
        if( !strcmp (name, "GREATER")) return SDLK_GREATER;
        if( !strcmp (name, "QUESTION")) return SDLK_QUESTION;
        if( !strcmp (name, "AT")) return SDLK_AT;
        if( !strcmp (name, "LEFTBRACKET")) return SDLK_LEFTBRACKET;
        if( !strcmp (name, "BACKSLASH")) return SDLK_BACKSLASH;
        if( !strcmp (name, "RIGHTBRACKET")) return SDLK_RIGHTBRACKET;
        if( !strcmp (name, "CARET")) return SDLK_CARET;
        if( !strcmp (name, "UNDERSCORE")) return SDLK_UNDERSCORE;
        if( !strcmp (name, "BACKQUOTE")) return SDLK_BACKQUOTE;
        if( !strcmp (name, "a")) return SDLK_a;
        if( !strcmp (name, "b")) return SDLK_b;
        if( !strcmp (name, "c")) return SDLK_c;
        if( !strcmp (name, "d")) return SDLK_d;
        if( !strcmp (name, "e")) return SDLK_e;
        if( !strcmp (name, "f")) return SDLK_f;
        if( !strcmp (name, "g")) return SDLK_g;
        if( !strcmp (name, "h")) return SDLK_h;
        if( !strcmp (name, "i")) return SDLK_i;
        if( !strcmp (name, "j")) return SDLK_j;
        if( !strcmp (name, "k")) return SDLK_k;
        if( !strcmp (name, "l")) return SDLK_l;
        if( !strcmp (name, "m")) return SDLK_m;
        if( !strcmp (name, "n")) return SDLK_n;
        if( !strcmp (name, "o")) return SDLK_o;
        if( !strcmp (name, "p")) return SDLK_p;
        if( !strcmp (name, "q")) return SDLK_q;
        if( !strcmp (name, "r")) return SDLK_r;
        if( !strcmp (name, "s")) return SDLK_s;
        if( !strcmp (name, "t")) return SDLK_t;
        if( !strcmp (name, "u")) return SDLK_u;
        if( !strcmp (name, "v")) return SDLK_v;
        if( !strcmp (name, "w")) return SDLK_w;
        if( !strcmp (name, "x")) return SDLK_x;
        if( !strcmp (name, "y")) return SDLK_y;
        if( !strcmp (name, "z")) return SDLK_z;
        if( !strcmp (name, "DELETE")) return SDLK_DELETE;
        if( !strcmp (name, "KP0")) return SDLK_KP0;
        if( !strcmp (name, "KP1")) return SDLK_KP1;
        if( !strcmp (name, "KP2")) return SDLK_KP2;
        if( !strcmp (name, "KP3")) return SDLK_KP3;
        if( !strcmp (name, "KP4")) return SDLK_KP4;
        if( !strcmp (name, "KP5")) return SDLK_KP5;
        if( !strcmp (name, "KP6")) return SDLK_KP6;
        if( !strcmp (name, "KP7")) return SDLK_KP7;
        if( !strcmp (name, "KP8")) return SDLK_KP8;
        if( !strcmp (name, "KP9")) return SDLK_KP9;
        if( !strcmp (name, "KP_PERIOD")) return SDLK_KP_PERIOD;
        if( !strcmp (name, "KP_DIVIDE")) return SDLK_KP_DIVIDE;
        if( !strcmp (name, "KP_MULTIPLY")) return SDLK_KP_MULTIPLY;
        if( !strcmp (name, "KP_MINUS")) return SDLK_KP_MINUS;
        if( !strcmp (name, "KP_PLUS")) return SDLK_KP_PLUS;
        if( !strcmp (name, "KP_ENTER")) return SDLK_KP_ENTER;
        if( !strcmp (name, "KP_EQUALS")) return SDLK_KP_EQUALS;
        if( !strcmp (name, "UP")) return SDLK_UP;
        if( !strcmp (name, "DOWN")) return SDLK_DOWN;
        if( !strcmp (name, "RIGHT")) return SDLK_RIGHT;
        if( !strcmp (name, "LEFT")) return SDLK_LEFT;
        if( !strcmp (name, "INSERT")) return SDLK_INSERT;
        if( !strcmp (name, "HOME")) return SDLK_HOME;
        if( !strcmp (name, "END")) return SDLK_END;
        if( !strcmp (name, "PAGEUP")) return SDLK_PAGEUP;
        if( !strcmp (name, "PAGEDOWN")) return SDLK_PAGEDOWN;
        if( !strcmp (name, "F0")) return SDLK_F10;
        if( !strcmp (name, "F1")) return SDLK_F1;
        if( !strcmp (name, "F2")) return SDLK_F2;
        if( !strcmp (name, "F3")) return SDLK_F3;
        if( !strcmp (name, "F4")) return SDLK_F4;
        if( !strcmp (name, "F5")) return SDLK_F5;
        if( !strcmp (name, "F6")) return SDLK_F6;
        if( !strcmp (name, "F7")) return SDLK_F7;
        if( !strcmp (name, "F8")) return SDLK_F8;
        if( !strcmp (name, "F9")) return SDLK_F9;
        if( !strcmp (name, "F10")) return SDLK_F10;
        if( !strcmp (name, "F11")) return SDLK_F11;
        if( !strcmp (name, "F12")) return SDLK_F12;
        if( !strcmp (name, "F13")) return SDLK_F13;
        if( !strcmp (name, "F14")) return SDLK_F14;
        if( !strcmp (name, "F15")) return SDLK_F15;
        if( !strcmp (name, "NUMLOCK")) return SDLK_NUMLOCK;
        if( !strcmp (name, "CAPSLOCK")) return SDLK_CAPSLOCK;
        if( !strcmp (name, "SCROLLOCK")) return SDLK_SCROLLOCK;
        if( !strcmp (name, "RSHIFT")) return SDLK_RSHIFT;
        if( !strcmp (name, "LSHIFT")) return SDLK_LSHIFT;
        if( !strcmp (name, "SHIFT")) return SDLK_LSHIFT; // for convenience
        if( !strcmp (name, "RCTRL")) return SDLK_RCTRL;
        if( !strcmp (name, "LCTRL")) return SDLK_LCTRL;
        if( !strcmp (name, "CTRL")) return SDLK_LCTRL; // // for convenience
        if( !strcmp (name, "RALT")) return SDLK_RALT;
        if( !strcmp (name, "LALT")) return SDLK_LALT;
        if( !strcmp (name, "RMETA")) return SDLK_RMETA;
        if( !strcmp (name, "LMETA")) return SDLK_LMETA;
        if( !strcmp (name, "LSUPER")) return SDLK_LSUPER;
        if( !strcmp (name, "RSUPER")) return SDLK_RSUPER;
        if( !strcmp (name, "MODE")) return SDLK_MODE;
        if( !strcmp (name, "HELP")) return SDLK_HELP;
        if( !strcmp (name, "PRINT")) return SDLK_PRINT;
        if( !strcmp (name, "SYSREQ")) return SDLK_SYSREQ;
        if( !strcmp (name, "BREAK")) return SDLK_BREAK;
        if( !strcmp (name, "MENU")) return SDLK_MENU;
        if( !strcmp (name, "POWER")) return SDLK_POWER;
        if( !strcmp (name, "EURO")) return SDLK_EURO;
        return -1;
}

int kbd_load_mapping_file(FILE *KbdFile)
{
        char     Buffer[512];
        char    *Scan;
        char    *sdlkName;

        while (fgets(Buffer,512,KbdFile) != (char *)0)
        {

                Scan = strchr(Buffer,'\n');
                if (Scan) *Scan = '\0';
                /* For this #@$% of windows ! */
                Scan = strchr(Buffer,'\r');
                if (Scan) *Scan = '\0';
                if (Buffer[0] == '#') continue;

                Scan = strchr(Buffer,'=');
                if (! Scan) continue;

                *Scan = '\0';
                sdlkName = Scan + 1;

                int gp2xButtonNum = buttonnameToGP2XButton(Buffer);
                printf ("gp2x = %s, sdlk = %s\n", Buffer, sdlkName);
                int sdlk = keynameToSDLK(sdlkName);
                apply_mapping(gp2xButtonNum, sdlk);
        }

        // now populate the lookup table
        gp2x_key_lookup[JOY_UP] = gp2xButtonToSDLK(GP2X_BUTTON_UP);
        gp2x_key_lookup[JOY_DOWN] = gp2xButtonToSDLK(GP2X_BUTTON_DOWN);
        gp2x_key_lookup[JOY_LEFT] = gp2xButtonToSDLK(GP2X_BUTTON_LEFT);
        gp2x_key_lookup[JOY_RIGHT] = gp2xButtonToSDLK(GP2X_BUTTON_RIGHT);
        gp2x_key_lookup[JOY_A] = gp2xButtonToSDLK(GP2X_BUTTON_A);
        gp2x_key_lookup[JOY_B] = gp2xButtonToSDLK(GP2X_BUTTON_B);
        gp2x_key_lookup[JOY_X] = gp2xButtonToSDLK(GP2X_BUTTON_X);
        gp2x_key_lookup[JOY_Y] = gp2xButtonToSDLK(GP2X_BUTTON_Y);
        gp2x_key_lookup[JOY_SELECT] = gp2xButtonToSDLK(GP2X_BUTTON_SELECT);
        return 0;
}

int kbd_load_mapping(char *kbd_filename)
{
        FILE    *KbdFile;
        int      error = 0;

        KbdFile = fopen(kbd_filename, "r");
        error   = 1;

        if (KbdFile != (FILE*)0)
        {
                kbd_load_mapping_file(KbdFile);
                error = 0;
                fclose(KbdFile);
        }

        return error;
}

void HandleButtonPress(int pressed, int joy)
{
        int col, row;

        int keysym = gp2x_key_lookup[joy];

        if (ConvertSDLKeyToBBCKey(keysym, &col, &row))
        {
                if (pressed)
                        BeebKeyDown(row, col);
                else
                        BeebKeyUp(row, col);
        }
}

void AllDirectionButtonsUp()
{
        int col, row;

        for (int i = 0; i < 4; i++)
        {
                ConvertSDLKeyToBBCKey(gp2x_key_lookup[i], &col, &row);
                BeebKeyUp(row, col);
        }
}

void HandleButtonPressForKeysym(int pressed, int keysym)
{
        int col, row;

        if (ConvertSDLKeyToBBCKey(keysym, &col, &row))
        {
                if (pressed)
                        BeebKeyDown(row, col);
                else
                        BeebKeyUp(row, col);
        }
}
